import logging
import time
import os
import re

from datetime import datetime

class Picker:

    directory = None
    dispatcher = None
    last_file_marker = None

    def __init__(self, directory, dispatcher):
        self.directory = directory
        self.dispatcher = dispatcher
        self.last_file_marker = None

    def read_last_file_marker(self):
        if self.last_file_marker is None:
            self.last_file_marker = datetime.now()
        return self.last_file_marker

    def update_last_file_marker(self, filename):
        filename = filename.rstrip(".m4a")
        marker = datetime.strptime(filename, "%Y_%m_%d_%H_%M_%S")
        self.last_file_marker = marker
        # TODO: also write to file, that one for later...

    def find_next_file(self, files, file_marker):
        for file in files:
            short = os.path.basename(file).rstrip(".m4a")
            marker = datetime.strptime(short, "%Y_%m_%d_%H_%M_%S")
            if file_marker < marker:
                return file

    def match_name(self, input):
        pattern = r"^\d{4}_\d{2}_\d{2}_\d{2}_\d{2}_\d{2}\.m4a$"
        return bool(re.match(pattern, input))

    def list_files(self):
        files = []
        for filename in os.listdir(self.directory):
            filepath = os.path.join(self.directory, filename)
            if os.path.isfile(filepath) and self.match_name(filename):
                files.append(filepath)
        return files

    def observe(self):
        logging.info(f"Watching directory: {self.directory}")
        while True:
            file_marker = self.read_last_file_marker()
            files = self.list_files()
            next_file = self.find_next_file(files, file_marker)
            if next_file is not None:
                self.dispatcher.process_audio_file(next_file)
                self.update_last_file_marker(os.path.basename(next_file).rstrip(".m4a"))
            time.sleep(10)
